// Gateway simulator.  (runs on com2)
//This program is used to simulate communication with the gateway mote.
//The best way to use this program is to have a null modem cable connected
//between com1 and com2 on your PC.  This way you can simulate communication
//between the gateway mote and wisedb.

#define STRICT
#include <tchar.h>
#include <windows.h>
#include <iostream>
#include <cstring>
#include <string.h>
#include <time.h>
#include "serial\Serial.h"
using namespace std;

CSerial serial;

bool checkerror(long lLastError, long error){			//Returns 0 upon success (no error)
		if (lLastError != ERROR_SUCCESS){
				cerr << "Win32 Error Code: " << error << endl;
				return 1;
		}
		return 0;
}

#define START_BYTE			0xa5
#define BROADCAST_ADDR		0xffff
#define PACKETTYPE_QUERY	0x70
#define PACKETTYPE_DATA		0x00
#define RESPONSECHAR			0x27

#define DATATYPE_LIGHT		0x01
#define DATATYPE_TEMP		0x02
#define DATATYPE_HUMID		0x03
#define DATATYPE_BATTERY	0x04

// Define the packet header structure:
#pragma pack(1)
typedef struct header_t {
	unsigned char	startbyte;
	unsigned short	addr;
	unsigned char	packettype;
	unsigned char	len;
} header_t;

#define	SIZEOF_HEADER		5

// Data structure:
typedef struct datapacket {
	unsigned char	datatype;
	unsigned short	datavalue;
} datapacket;

#pragma pack(8)

#define	SIZEOF_DATAPACKET	3

//packet assembler function
unsigned char *assemblepacket(unsigned char packettype,
						unsigned short address,
						unsigned short moteid,
						unsigned char &len,
						unsigned short args, ...)
{
	va_list	ap;

	//here is the packet straucture  start 
	//	startbyte(1), address(2), type(1), datalength(1), data([datalength])

	unsigned char *packet;

	len = args * SIZEOF_DATAPACKET + SIZEOF_HEADER + (args != 0 ? sizeof(unsigned short) : 0);
	packet = new unsigned char[len];

	// now it dont matter what kinda packet u send.
	// this will fill out the header info no matter what.
	// and if its a data packet (i.e., args != 0) then
	// it will stuff that data into the end of the packet
	// and calculate the correct length of the total packet
	// and return everything.
	header_t	*hdr;
	hdr = (header_t *)&(packet[0]);

	hdr->startbyte = START_BYTE;			//start byte
	hdr->addr = address;
	hdr->packettype = packettype;			//packet type
	hdr->len = len - SIZEOF_HEADER;		//length of data field

	if (args > 0) {
		// Put moteid as an unsigned short into packet[5]:
		*((unsigned short *)&(packet[SIZEOF_HEADER])) = moteid;
		
		// good ol' fashioned trick:
		// make a datapacket pointer.
		// then assign it to the memory where data[] is supposed to be
		// in the packet. [byte 7]:
		datapacket *data;
		data = (datapacket *) &(packet[SIZEOF_HEADER + sizeof(unsigned short)]);

		va_start(ap, args);
		// args is the number of argument PAIRS ((datatype, datavalue) = 1 pair) after c in the function call:
		// fill up the data[] array with the arguments:
		while (args--) {
			// Grab datatype number and then the datavalue.
			data->datatype = va_arg(ap, unsigned char);		// Data type
			data->datavalue = va_arg(ap, unsigned short);
			// move to the next position to write to in the data structure:
			data++;
		};
		va_end(ap);
	};
	
	int i;
	for (i = 0; i < len; i++) {
		printf("0x%02x, ", packet[i]);
	};
	printf("\n");

	return packet;
}

unsigned char *getpacket() {
	header_t	header;		// temporary header to read in data
								// so we can allocate appropriate amount
								// of memory for "char *packet"

	unsigned char	*packet = NULL;
	
	serial.Read(&header.startbyte, 1);
	if (header.startbyte != START_BYTE) return NULL;

	printf("Reading header (%d bytes)...\n", SIZEOF_HEADER - sizeof(unsigned char));

	// Read the REST of the header after startbyte:
	serial.Read(&header.addr, sizeof(unsigned short));
	serial.Read(&header.packettype, sizeof(unsigned char));
	serial.Read(&header.len, sizeof(unsigned char));

	printf("Received rest of header\n");

	// Allocate memory for packet (header size + length of data):
	packet = new unsigned char[SIZEOF_HEADER + header.len];

	// Copy data from TEMPORARY header into packet[]
	memcpy(packet, &header, SIZEOF_HEADER);

	printf("Header:\n");
	printf("  startbyte  = 0x%02x\n", header.startbyte);
	printf("  address    = 0x%04x\n", header.addr);
	printf("  packettype = 0x%02x\n", header.packettype);
	printf("  len        = %d\n", header.len);

	printf("Receiveing data... (%d bytes)", header.len);
	// Read the rest of the packet data into the packet:
	serial.Read(&(packet[SIZEOF_HEADER]), header.len);

	printf("done.\n");

	return packet;
};


int main ()
{
	LONG    lLastError = ERROR_SUCCESS;

	// Attempt to open the serial port (COM2)
	lLastError = serial.Open(_T("COM2"));
	checkerror(lLastError,serial.GetLastError());

	// Setup the serial port (9600,N81) using hardware handshaking
	lLastError = serial.Setup(CSerial::EBaud9600,CSerial::EData8,CSerial::EParNone,CSerial::EStop1);
	checkerror(lLastError,serial.GetLastError());

	// Setup handshaking (none)
	lLastError = serial.SetupHandshaking(CSerial::EHandshakeHardware);
	checkerror(lLastError,serial.GetLastError());

	//Setup read timeouts (blocking - wait until data received.)
	lLastError = serial.SetupReadTimeouts(CSerial::EReadTimeoutBlocking);
	// The serial port is now ready and we can send/receive data. If
	// the following call blocks, then the other side doesn't support
	// hardware handshaking.

	// Use 'non-blocking' reads, because we don't know how many bytes
	// will be received. This is normally the most convenient mode
	// (and also the default mode for reading data).

	//----------------------start of communications----------------
	int moteid;
	char description[50] = "\"light\"";
	unsigned char *packet;
	unsigned char responsechar = RESPONSECHAR;
	unsigned char	packetlen;
	header_t			*hdr;

	for(;;) {		//overall loop
		for (;;)	{
			packet = getpacket();
			if (packet == NULL) { cout<<"Packet corrupt (did not receive correct startbyte)!"<<endl; continue; };
			
			hdr = (header_t *) &(packet[0]);
			if (hdr->packettype == PACKETTYPE_DATA) { delete [] packet; continue; };
			if (hdr->packettype == PACKETTYPE_QUERY) break;
			delete [] packet;
		}

		cout << "Query heard!" << endl << endl;
		serial.Write(&responsechar,1);  //This sends the ack response

		// Remove old QUERY packet from memory:
		delete [] packet;			// prevent memory leak:

		moteid = rand() % 1000;

		cout << "Sending data packet:" << endl;
		cout << "  from moteid:	" << moteid << endl;
		
		packet = assemblepacket(PACKETTYPE_DATA, 0x0000, moteid, packetlen, 4,
				DATATYPE_LIGHT, rand() % 100,
				DATATYPE_TEMP, rand() % 100,
				DATATYPE_HUMID, rand() % 100,
				DATATYPE_BATTERY, rand() % 100);

		hdr = (header_t *) &(packet[0]);
		printf("Header:\n");
		printf("  startbyte  = 0x%02x\n", hdr->startbyte);
		printf("  address    = 0x%04x\n", hdr->addr);
		printf("  packettype = 0x%02x\n", hdr->packettype);
		printf("  len        = %d\n", hdr->len);
		
		// Write the packet:
		serial.Write(packet, packetlen);

		delete [] packet;

/*		size = strlen(description);
		serial.Write(&size,sizeof(size));
		serial.Write((LPCSTR)description);
*/

		//Initialize read.
		//Setup read timeouts (blocking - wait until data received.)
		lLastError = serial.SetupReadTimeouts(CSerial::EReadTimeoutBlocking);
		checkerror(lLastError,serial.GetLastError());

		//create time delay seconds.
		clock_t start = clock();
		while ((clock() - start) < 15 * CLOCKS_PER_SEC);		//delay 15 seconds.


	}	//end overall loop.


    // Close the port again
    serial.Close();
    return 0;
}



